@@ -26,6 +26,7 @@ require 'graphviz/edge'
 require 'graphviz/attrs'
 require 'graphviz/constants'
 require 'graphviz/elements'
+require 'graphviz/dot_script'
 
 require 'graphviz/dot2ruby'
 require 'graphviz/types'
@@ -36,38 +37,38 @@ if /^1.8/.match RUBY_VERSION
 end
 
 class GraphViz
-  include Constants
-  include GVUtils
+  include GraphViz::Constants
+  include GraphViz::Utils
 
   public
 
   ## Var: Output format (dot, png, jpeg, ...)
   @@format = nil #"canon"
-  @format
+  @format = nil
   ## Var: Output file name
-  @filename
+  @filename = nil
   ## Var: Output format and file
-  @output
+  @output = nil
   ## Var: program to use (dot|twopi)
   @@prog = "dot"
-  @prog
+  @prog = nil
   ## Var: program path
   @@path = []
-  @path
+  @path = nil
   ## Var: Error level
   @@errors = 1
-  @errors
+  @errors = nil
   ## Var: External libraries
   @@extlibs = []
-  @extlibs
+  @extlibs = nil
 
   ## Var: Graph name
-  @name
+  @name = nil
 
   ## Var: defined attributes
-  @graph
-  @node
-  @edge
+  @graph = nil
+  @node = nil
+  @edge = nil
 
   # This accessor allow you to set global graph attributes
   attr_accessor :graph
@@ -81,12 +82,14 @@ class GraphViz
   attr_accessor :edge
   alias_method :edge_attrs, :edge
 
-  @elements_order
 
-  #<b>DEPRECATED</b> please use GraphViz#add_nodes
+  @elements_order = nil
+
   def add_node( xNodeName, hOpts = {} )
-     warn "GraphViz#add_node is deprecated, please use GraphViz#add_nodes"
-     return add_nodes(xNodeName, hOpts)
+    if xNodeName.kind_of? Array
+      raise ArgumentError, "use `add_nodes' to add several nodes at the same time"
+    end
+    return add_nodes(xNodeName, hOpts)
   end
 
   # Create a new node
@@ -179,10 +182,11 @@ class GraphViz
     @hoNodes.size
   end
 
-  #<b>DEPRECATED</b> please use GraphViz#add_edges
   def add_edge( oNodeOne, oNodeTwo, hOpts = {} )
-     warn "GraphViz#add_edge is deprecated, please use GraphViz#add_edges"
-     add_edges(oNodeOne, oNodeTwo, hOpts)
+    if oNodeTwo.kind_of? Array or oNodeOne.kind_of? Array 
+      raise ArgumentError, "use `add_edges' to add several edges at the same time"
+    end
+    add_edges(oNodeOne, oNodeTwo, hOpts)
   end
 
   # Create a new edge
@@ -387,7 +391,12 @@ class GraphViz
         self[key] = value
       end
     else
-      return( @graph[xAttrName].clone )
+      attr = @graph[xAttrName]
+      if attr.nil?
+        return nil
+      else
+        return( @graph[xAttrName].clone )
+      end
     end
   end
 
@@ -405,13 +414,20 @@ class GraphViz
      each_attribute(&b)
   end
 
+  # Create a new graph from the current subgraph
+  def to_graph
+    graph = self.clone
+    graph.pg = nil
+    return graph
+  end
+
   #
   # Generate the graph
   #
   # Options :
-  # * :output : Output format (Constants::FORMATS)
+  # * :output : Output format (GraphViz::Constants::FORMATS)
   # * :file : Output file name
-  # * :use : Program to use (Constants::PROGRAMS)
+  # * :use : Program to use (GraphViz::Constants::PROGRAMS)
   # * :path : Program PATH
   # * :<format> => <file> : <file> can be
   #   * a file name
@@ -423,86 +439,15 @@ class GraphViz
   #   * 2 = none
   #
   def output( hOpts = {} )
-    xDOTScript = ""
-    xLastType = nil
-    xSeparator = ""
-    xData = ""
+    xDOTScript = DOTScript.new
     lNotHugly = []
 
-    @elements_order.each { |kElement|
-      if xLastType.nil? or xLastType != kElement["type"]
+    append_attributes_and_types(xDOTScript)
 
-        if xData.length > 0
-          case xLastType
-            when "graph_attr"
-              xDOTScript << "  " + xData + ";\n"
-
-            when "node_attr"
-              xDOTScript << "  node [" + xData + "];\n"
-
-            when "edge_attr"
-              xDOTScript << "  edge [" + xData + "];\n"
-          end
-        end
-
-        xSeparator = ""
-        xData = ""
-      end
-
-      xLastType = kElement["type"]
-
-      #Modified by
-      #Brandon Coleman
-      #verify value is NOT NULL
-      if kElement["value"] == nil then
-        raise ArgumentError, "#{kElement["name"]} has a nil value!"
-      end
-
-      case kElement["type"]
-        when "graph_attr"
-          xData << xSeparator + kElement["name"] + " = " + kElement["value"].to_gv
-          xSeparator = "; "
-
-        when "node_attr"
-          xData << xSeparator + kElement["name"] + " = " + kElement["value"].to_gv
-          xSeparator = ", "
-
-        when "edge_attr"
-          xData << xSeparator + kElement["name"] + " = " + kElement["value"].to_gv
-          xSeparator = ", "
-
-        when "node"
-          xDOTScript << "  " + kElement["value"].output() + "\n"
-
-        when "edge"
-          xDOTScript << "  " + kElement["value"].output( @oGraphType ) + "\n"
-
-        when "graph"
-          xDOTScript << kElement["value"].output() + "\n"
-
-        else
-          raise ArgumentError, "Don't know what to do with element type '#{kElement['type']}'"
-      end
-    }
-
-    if xData.length > 0
-      case xLastType
-        when "graph_attr"
-          xDOTScript << "  " + xData + ";\n"
-
-        when "node_attr"
-          xDOTScript << "  node [" + xData + "];\n"
-
-        when "edge_attr"
-          xDOTScript << "  edge [" + xData + "];\n"
-      end
-    end
     xDOTScript << "}"
 
-    if @oParentGraph
-      xDOTScript = "subgraph #{GraphViz.escape(@name, :unquote_empty => true)} {\n" << xDOTScript
-
-      return( xDOTScript )
+    if has_parent_graph?
+      xDOTScript.make_subgraph("#{GraphViz.escape(@name, :unquote_empty => true)}")
     else
       hOutput = {}
 
@@ -554,7 +499,7 @@ class GraphViz
             begin
               require 'graphviz/nothugly'
               @nothugly = true
-            rescue LoadError => e
+            rescue LoadError
               warn "You must install ruby-xslt or libxslt-ruby to use nothugly option!"
               @nothugly = false
             end
@@ -569,7 +514,9 @@ class GraphViz
       @output = hOutput if hOutput.size > 0
 
       xStict = ((@strict && @oGraphType == "digraph") ? "strict " : "")
-      xDOTScript = ("#{xStict}#{@oGraphType} #{GraphViz.escape(@name, :unquote_empty => true)} {\n" << xDOTScript).gsub( "\0", "" )
+      xDOTScript.prepend(
+        "#{xStict}#{@oGraphType} #{GraphViz.escape(@name, :unquote_empty => true)} {"
+      )
 
       xOutputString = (@filename == String ||
         @output.any? {|format, file| file == String })
@@ -597,61 +544,59 @@ class GraphViz
           raise StandardError, "GraphViz not installed or #{@prog} not in PATH. Install GraphViz or use the 'path' option"
         end
 
-        cmd = escape_path_containing_blanks(cmd) if IS_JRUBY
-
-        xOutputWithFile = ""
-        xOutputWithoutFile = ""
+        xOutputWithFile = []
+        xOutputWithoutFile = []
         unless @format.nil? or @format == "none"
           lNotHugly << @filename if @format.to_s == "svg" and @nothugly
           if @filename.nil? or @filename == String
-            xOutputWithoutFile = "-T#{@format} "
+            xOutputWithoutFile = ["-T#{@format}"]
           else
-            xOutputWithFile = "-T#{@format} -o#{@filename} "
+            xOutputWithFile = ["-T#{@format}", "-o#{@filename}"]
           end
         end
         @output.each_except( :key => ["none"] ) do |format, file|
           lNotHugly << file if format.to_s == "svg" and @nothugly
           if file.nil? or file == String
-            xOutputWithoutFile << "-T#{format} "
+            xOutputWithoutFile += ["-T#{format}"]
           else
-            xOutputWithFile << "-T#{format} -o#{file} "
+            xOutputWithFile += ["-T#{format}", "-o#{file}"]
           end
         end
 
-        xExternalLibraries = ""
-        @extlibs.each do |lib|
-          xExternalLibraries << "-l#{lib} "
-        end
+        xExternalLibraries = @extlibs.map { |lib| "-l#{lib}" }
 
-        xOtherOptions = ""
-        xOtherOptions += " -s#{@scale}" unless @scale.nil?
-        xOtherOptions += " -y" if @inverty
-        unless @no_layout.nil?
-          xOtherOptions += " -n"
-          xOtherOptions += "2" if @no_layout.to_i == 2
-        end
-        xOtherOptions += " -x" if @reduce_graph
-        xOtherOptions += " -Lg" if @Lg
-        xOtherOptions += " -LO" if @LO
-        xOtherOptions += " -Ln#{@Ln}" unless @LN.nil?
-        xOtherOptions += " -LU#{@LU}" unless @LU.nil?
-        xOtherOptions += " -LC#{@LC}" unless @LC.nil?
-        xOtherOptions += " -LT#{@LT}" unless @LT.nil?
+        xOtherOptions = []
+        xOtherOptions << "-s#{@scale}" if @scale
+        xOtherOptions << "-y" if @inverty
+        xOtherOptions << "-n#{@no_layout}" if @no_layout
+        xOtherOptions << "-x" if @reduce_graph
+        xOtherOptions << "-Lg" if @Lg
+        xOtherOptions << "-LO" if @LO
+        xOtherOptions << "-Ln#{@Ln}" if @Ln
+        xOtherOptions << "-LU#{@LU}" if @LU
+        xOtherOptions << "-LC#{@LC}" if @LC
+        xOtherOptions << "-LT#{@LT}" if @LT
 
-        if IS_JRUBY
-          xCmd = "#{cmd} -q#{@errors} #{xExternalLibraries} #{xOtherOptions} #{xOutputWithFile} #{xOutputWithoutFile} #{t.path}"
+        tmpPath = if IS_JRUBY
+          t.path
         elsif IS_CYGWIN
-          tmpPath = t.path
           begin
-            tmpPath = "'" + `cygpath -w #{t.path}`.chomp + "'"
+            IO.popen("cygpath", "-w", t.path).chomp
           rescue
             warn "cygpath is not installed!"
+            t.path
           end
-          xCmd = "\"#{cmd}\" -q#{@errors} #{xExternalLibraries} #{xOtherOptions} #{xOutputWithFile} #{xOutputWithoutFile} #{tmpPath}"
         else
-          xCmd = "\"#{cmd}\" -q#{@errors} #{xExternalLibraries} #{xOtherOptions} #{xOutputWithFile} #{xOutputWithoutFile} #{t.path}"
+          t.path
         end
 
+        xCmd = [cmd, "-q#{@errors}"] +
+               xExternalLibraries +
+               xOtherOptions +
+               xOutputWithFile +
+               xOutputWithoutFile +
+               [tmpPath]
+
         xOutput << output_from_command( xCmd )
       end
 
@@ -671,9 +616,36 @@ class GraphViz
       end
     end
   end
-
   alias :save :output
 
+  def append_attributes_and_types(script)
+    script_data = DOTScriptData.new
+
+    @elements_order.each { |kElement|
+      is_new_type = script_data.type != kElement["type"]
+      if is_new_type 
+        script << script_data unless script_data.type.nil? or script_data.empty?
+        script_data = DOTScriptData.new(kElement["type"])
+      end
+
+      # Modified by Brandon Coleman verify value is NOT NULL
+      kElement["value"] or raise ArgumentError, "#{kElement["name"]} is nil!"
+
+      case kElement["type"]
+        when "graph_attr", "node_attr", "edge_attr"
+          script_data.add_attribute(kElement["name"], kElement["value"].to_gv)
+        when "node", "graph"
+          script << kElement["value"].output()
+        when "edge"
+          script << "  " + kElement["value"].output( @oGraphType )
+        else
+          raise ArgumentError,
+            "Don't know what to do with element type '#{kElement['type']}'"
+      end
+    }
+    script << script_data unless script_data.type.nil? or script_data.empty?
+  end
+
   def to_s
     self.output(:none => String)
   end
@@ -755,7 +727,7 @@ class GraphViz
         when "extlibs"
           @@extlibs = v.split( "," ).map{ |x| x.strip }
         else
-          warn "Invalide option #{k}!"
+          warn "Invalid option #{k}!"
       end
     end
   end
@@ -769,9 +741,9 @@ class GraphViz
   # Create a new graph from a GraphViz File
   #
   # Options :
-  # * :output : Output format (Constants::FORMATS) (default : dot)
+  # * :output : Output format (GraphViz::Constants::FORMATS) (default : dot)
   # * :file : Output file name (default : none)
-  # * :use : Program to use (Constants::PROGRAMS) (default : dot)
+  # * :use : Program to use (GraphViz::Constants::PROGRAMS) (default : dot)
   # * :path : Program PATH
   #
   def self.parse( xFile, hOpts = {}, &block )
@@ -783,14 +755,14 @@ class GraphViz
   # Create a new graph from a GraphViz File
   #
   # Options :
-  # * :output : Output format (Constants::FORMATS) (default : dot)
+  # * :output : Output format (GraphViz::Constants::FORMATS) (default : dot)
   # * :file : Output file name (default : none)
-  # * :use : Program to use (Constants::PROGRAMS) (default : dot)
+  # * :use : Program to use (GraphViz::Constants::PROGRAMS) (default : dot)
   # * :path : Program PATH
   #
   def self.parse_string( str, hOpts = {}, &block )
-    graph = Dot2Ruby::new( hOpts[:path], nil, nil ).eval_string( str )
-    yield( graph ) if( block and graph )
+    graph = Dot2Ruby::new(hOpts[:path], nil, nil).eval_string(str)
+    yield(graph) if(block and graph)
     return graph
   end
 
@@ -807,33 +779,37 @@ class GraphViz
 
   # Return true if the graph is directed.
   def directed?
-     not (/digraph/ =~ "bla digraph bla").nil?
+    not((/digraph/ =~ "bla digraph bla").nil?)
+  end
+
+  def has_parent_graph?
+     not @oParentGraph.nil?
   end
 ## ----------------------------------------------------------------------------
 
   private
 
   ## Var: Nodes, Edges and Graphs tables
-  @hoNodes
-  @loEdges
-  @hoGraphs
+  @hoNodes = nil
+  @loEdges = nil
+  @hoGraphs = nil
 
   ## Var: Parent graph
-  @oParentGraph
+  @oParentGraph = nil
 
   ## Var: Type de graphe (orienté ou non)
-  @oGraphType
+  @oGraphType = nil
 
   #
   # Create a new graph object
   #
   # Options :
-  # * :output : Output format (Constants::FORMATS) (default : dot)
+  # * :output : Output format (GraphViz::Constants::FORMATS) (default : dot)
   # * :file : Output file name (default : nil)
-  # * :use : Program to use (Constants::PROGRAMS) (default : dot)
+  # * :use : Program to use (GraphViz::Constants::PROGRAMS) (default : dot)
   # * :path : Program PATH
   # * :parent : Parent graph (default : nil)
-  # * :type : Graph type (Constants::GRAPHTYPE) (default : digraph)
+  # * :type : Graph type (GraphViz::Constants::GRAPHTYPE) (default : digraph)
   # * :errors : DOT error level (default 1)
   #   * 0 = Error + Warning
   #   * 1 = Error
@@ -953,7 +929,7 @@ class GraphViz
      end
 
      nodes = (1..num_nodes).map{ |e| e.to_s }
-     x = g.add_nodes(nodes)
+     g.add_nodes(nodes)
 
      edges = []
      nodes.each do |head|
@@ -981,7 +957,8 @@ class GraphViz
       :unquote_empty => false,
     }.merge(opts)
 
-    if (options[:force] or str.match( /\A[a-zA-Z_]+[a-zA-Z0-9_]*\Z/ ).nil?)
+    reserved_words = %w{node edge graph digraph subgraph strict}
+    if (options[:force] or str.match( /\A[a-zA-Z_]+[a-zA-Z0-9_]*\Z/ ).nil? or reserved_words.include?(str.downcase))
       unless options[:unquote_empty] and str.size == 0
         '"' + str.gsub('"', '\\"').gsub("\n", '\\\\n') + '"'
       end
