@@ -1,7 +1,4 @@
-require 'test/unit'
-$:.unshift(File.expand_path('../../lib',__FILE__))
-require 'graphviz'
-require 'graphviz/theory'
+require 'helper'
 
 class GraphVizSearch < Test::Unit::TestCase
    def setup
